@@ -32,6 +32,7 @@
 
 #include <netinet/in.h>
 
+#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -298,7 +299,8 @@ dhcp6_updateelapsed(struct interface *ifp, struct dhcp6_message *m, size_t len)
 	struct dhcp6_state *state;
 	const struct dhcp6_option *co;
 	struct dhcp6_option *o;
-	time_t up;
+	struct timespec tv;
+	time_t hsec;
 	uint16_t u16;
 
 	co = dhcp6_getmoption(D6_OPTION_ELAPSED, m, len);
@@ -307,10 +309,26 @@ dhcp6_updateelapsed(struct interface *ifp, struct dhcp6_message *m, size_t len)
 
 	o = __UNCONST(co);
 	state = D6_STATE(ifp);
-	up = uptime() - state->start_uptime;
-	if (up < 0 || up > (time_t)UINT16_MAX)
-		up = (time_t)UINT16_MAX;
-	u16 = htons((uint16_t)up);
+	clock_gettime(CLOCK_MONOTONIC, &tv);
+	if (state->RTC == 0) {
+		/* An RTC of zero means we're the first message
+		 * out of the door, so the elapsed time is zero. */
+		state->started = tv;
+		hsec = 0;
+	} else {
+		timespecsub(&tv, &state->started, &tv);
+		/* Elapsed time is measured in centiseconds.
+		 * We need to be sure it will not potentially overflow. */
+		if (tv.tv_sec >= (UINT16_MAX / CSEC_PER_SEC) + 1)
+			hsec = UINT16_MAX;
+		else {
+			hsec = (tv.tv_sec * CSEC_PER_SEC) +
+			    (tv.tv_nsec / NSEC_PER_CSEC);
+			if (hsec > UINT16_MAX)
+				hsec = UINT16_MAX;
+		}
+	}
+	u16 = htons((uint16_t)hsec);
 	memcpy(D6_OPTION_DATA(o), &u16, sizeof(u16));
 	return 0;
 }
@@ -642,31 +660,20 @@ dhcp6_makemessage(struct interface *ifp)
 			    !(ap->flags & IPV6_AF_REQUEST))
 				continue;
 			if (ap->ia_type == D6_OPTION_IA_PD) {
-				if (!(ifo->options & DHCPCD_NOPFXDLG)) {
-					len += sizeof(*o) + sizeof(u8) +
-					    sizeof(u32) + sizeof(u32) +
-					    sizeof(ap->prefix);
-					if (ap->prefix_exclude_len)
-						len += sizeof(*o) + 1 +
-						    (uint8_t)((ap->prefix_exclude_len -
-						    ap->prefix_len - 1) / NBBY)
-						    + 1;
-
-				}
-			} else if (!(ifo->options & DHCPCD_PFXDLGONLY))
+				len += sizeof(*o) + sizeof(u8) +
+				    sizeof(u32) + sizeof(u32) +
+				    sizeof(ap->prefix);
+				if (ap->prefix_exclude_len)
+					len += sizeof(*o) + 1 +
+					    (uint8_t)((ap->prefix_exclude_len -
+					    ap->prefix_len - 1) / NBBY) + 1;
+			} else
 				len += sizeof(*o) + sizeof(ap->addr) +
 				    sizeof(u32) + sizeof(u32);
 		}
 		/* FALLTHROUGH */
 	case DH6S_INIT:
-		for (l = 0; l < ifo->ia_len; l++) {
-			if (ifo->ia[l].ia_type == D6_OPTION_IA_PD) {
-				if (ifo->options & DHCPCD_NOPFXDLG)
-					continue;
-			} else if (ifo->options & DHCPCD_PFXDLGONLY)
-				continue;
-			len += sizeof(*o) + (sizeof(u32) * 3);
-		}
+		len += ifo->ia_len * (sizeof(*o) + (sizeof(u32) * 3));
 		IA = 1;
 		break;
 	default:
@@ -778,11 +785,6 @@ dhcp6_makemessage(struct interface *ifp)
 	}
 
 	for (l = 0; IA && l < ifo->ia_len; l++) {
-		if (ifo->ia[l].ia_type == D6_OPTION_IA_PD) {
-			if (ifo->options & DHCPCD_NOPFXDLG)
-				continue;
-		} else if (ifo->options & DHCPCD_PFXDLGONLY)
-			continue;
 		o = D6_NEXT_OPTION(o);
 		o->code = htons(ifo->ia[l].ia_type);
 		o->len = htons(sizeof(u32) + sizeof(u32) + sizeof(u32));
@@ -1019,6 +1021,9 @@ dhcp6_sendmessage(struct interface *ifp, void (*callback)(void *))
 	const char *broad_uni;
 	const struct in6_addr alldhcp = IN6ADDR_LINKLOCAL_ALLDHCP_INIT;
 
+	if (!callback && ifp->carrier == LINK_DOWN)
+		return 0;
+
 	memset(&dst, 0, sizeof(dst));
 	dst.sin6_family = AF_INET6;
 	dst.sin6_port = htons(DHCP6_SERVER_PORT);
@@ -1051,6 +1056,9 @@ dhcp6_sendmessage(struct interface *ifp, void (*callback)(void *))
 		    state->send->xid[1],
 		    state->send->xid[2]);
 	else {
+		if (state->IMD &&
+		    !(ifp->options->options & DHCPCD_INITIAL_DELAY))
+			state->IMD = 0;
 		if (state->IMD) {
 			/* Some buggy PPP servers close the link too early
 			 * after sending an invalid status in their reply
@@ -1091,7 +1099,7 @@ dhcp6_sendmessage(struct interface *ifp, void (*callback)(void *))
 		else
 			timespecadd(&state->RT, &RTprev, &state->RT);
 
-		if (state->RT.tv_sec > state->MRT) {
+		if (state->MRT != 0 && state->RT.tv_sec > state->MRT) {
 			RTprev.tv_sec = state->MRT;
 			RTprev.tv_nsec = 0;
 			state->RT.tv_sec = state->MRT;
@@ -1106,19 +1114,24 @@ dhcp6_sendmessage(struct interface *ifp, void (*callback)(void *))
 		}
 
 logsend:
-		logger(ifp->ctx, LOG_DEBUG,
-		    "%s: %s %s (xid 0x%02x%02x%02x),"
-		    " next in %0.1f seconds",
-		    ifp->name,
-		    broad_uni,
-		    dhcp6_get_op(state->send->type),
-		    state->send->xid[0],
-		    state->send->xid[1],
-		    state->send->xid[2],
-		    timespec_to_double(&state->RT));
+		if (ifp->carrier != LINK_DOWN)
+			logger(ifp->ctx, LOG_DEBUG,
+			    "%s: %s %s (xid 0x%02x%02x%02x),"
+			    " next in %0.1f seconds",
+			    ifp->name,
+			    broad_uni,
+			    dhcp6_get_op(state->send->type),
+			    state->send->xid[0],
+			    state->send->xid[1],
+			    state->send->xid[2],
+			    timespec_to_double(&state->RT));
+
+		/* This sometimes happens when we delegate to this interface
+		 * AND run DHCPv6 on it normally. */
+		assert(timespec_to_double(&state->RT) != 0);
 
 		/* Wait the initial delay */
-		if (state->IMD) {
+		if (state->IMD != 0) {
 			state->IMD = 0;
 			eloop_timeout_add_tv(ifp->ctx->eloop,
 			    &state->RT, callback, ifp);
@@ -1126,6 +1139,9 @@ logsend:
 		}
 	}
 
+	if (ifp->carrier == LINK_DOWN)
+		return 0;
+
 	/* Update the elapsed time */
 	dhcp6_updateelapsed(ifp, state->send, state->send_len);
 	if (ifp->options->auth.options & DHCPCD_AUTH_SEND &&
@@ -1220,14 +1236,12 @@ dhcp6_sendconfirm(void *arg)
 	dhcp6_sendmessage(arg, dhcp6_sendconfirm);
 }
 
-/*
 static void
 dhcp6_sendrelease(void *arg)
 {
 
 	dhcp6_sendmessage(arg, dhcp6_sendrelease);
 }
-*/
 
 static void
 dhcp6_startrenew(void *arg)
@@ -1238,7 +1252,6 @@ dhcp6_startrenew(void *arg)
 	ifp = arg;
 	state = D6_STATE(ifp);
 	state->state = DH6S_RENEW;
-	state->start_uptime = uptime();
 	state->RTC = 0;
 	state->IRT = REN_TIMEOUT;
 	state->MRT = REN_MAX_RT;
@@ -1378,7 +1391,6 @@ dhcp6_startdiscover(void *arg)
 	logger(ifp->ctx, LOG_INFO, "%s: soliciting a DHCPv6 lease", ifp->name);
 	state = D6_STATE(ifp);
 	state->state = DH6S_DISCOVER;
-	state->start_uptime = uptime();
 	state->RTC = 0;
 	state->IMD = SOL_MAX_DELAY;
 	state->IRT = SOL_TIMEOUT;
@@ -1452,9 +1464,6 @@ dhcp6_hasprefixdelegation(struct interface *ifp)
 	size_t i;
 	uint16_t t;
 
-	if (ifp->options->options & DHCPCD_NOPFXDLG)
-		return 0;
-
 	t = 0;
 	for (i = 0; i < ifp->options->ia_len; i++) {
 		if (t && t != ifp->options->ia[i].ia_type) {
@@ -1541,7 +1550,6 @@ dhcp6_startconfirm(struct interface *ifp)
 
 	state = D6_STATE(ifp);
 	state->state = DH6S_CONFIRM;
-	state->start_uptime = uptime();
 	state->RTC = 0;
 	state->IMD = CNF_MAX_DELAY;
 	state->IRT = CNF_TIMEOUT;
@@ -1572,7 +1580,6 @@ dhcp6_startinform(void *arg)
 		logger(ifp->ctx, LOG_INFO,
 		    "%s: requesting DHCPv6 information", ifp->name);
 	state->state = DH6S_INFORM;
-	state->start_uptime = uptime();
 	state->RTC = 0;
 	state->IMD = INF_MAX_DELAY;
 	state->IRT = INF_TIMEOUT;
@@ -1605,6 +1612,18 @@ dhcp6_startexpire(void *arg)
 		    "%s: no advertising IPv6 router wants DHCP", ifp->name);
 }
 
+static void
+dhcp6_finishrelease(void *arg)
+{
+	struct interface *ifp;
+	struct dhcp6_state *state;
+
+	ifp = (struct interface *)arg;
+	state = D6_STATE(ifp);
+	state->state = DH6S_RELEASED;
+	dhcp6_drop(ifp, "RELEASE6");
+}
+
 static void
 dhcp6_startrelease(struct interface *ifp)
 {
@@ -1615,22 +1634,25 @@ dhcp6_startrelease(struct interface *ifp)
 		return;
 
 	state->state = DH6S_RELEASE;
-	state->start_uptime = uptime();
 	state->RTC = 0;
 	state->IRT = REL_TIMEOUT;
 	state->MRT = 0;
+	/* MRC of REL_MAX_RC is optional in RFC 3315 18.1.6 */
+#if 0
 	state->MRC = REL_MAX_RC;
-	//state->MRCcallback = dhcp6_failrelease;
+	state->MRCcallback = dhcp6_finishrelease;
+#else
+	state->MRC = 0;
 	state->MRCcallback = NULL;
+#endif
 
 	if (dhcp6_makemessage(ifp) == -1)
 		logger(ifp->ctx, LOG_ERR,
 		    "%s: dhcp6_makemessage: %m", ifp->name);
-	else
-		/* XXX: We should loop a few times
-		 * Luckily RFC3315 section 18.1.6 says this is optional */
-		//dhcp6_sendrelease(ifp);
-		dhcp6_sendmessage(ifp, NULL);
+	else {
+		dhcp6_sendrelease(ifp);
+		dhcp6_finishrelease(ifp);
+	}
 }
 
 static int
@@ -1686,24 +1708,16 @@ dhcp6_checkstatusok(const struct interface *ifp,
 	return -1;
 }
 
-static struct ipv6_addr *
-dhcp6_iffindaddr(struct interface *ifp, const struct in6_addr *addr,
+const struct ipv6_addr *
+dhcp6_iffindaddr(const struct interface *ifp, const struct in6_addr *addr,
     short flags)
 {
-	struct dhcp6_state *state;
-	struct ipv6_addr *ap;
+	const struct dhcp6_state *state;
+	const struct ipv6_addr *ap;
 
-	state = D6_STATE(ifp);
-	if (state) {
+	if ((state = D6_STATE(ifp)) != NULL) {
 		TAILQ_FOREACH(ap, &state->addrs, next) {
-			if (addr == NULL) {
-				if ((ap->flags &
-				    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED)) ==
-				    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED))
-					return ap;
-			} else if (ap->prefix_vltime &&
-			    IN6_ARE_ADDR_EQUAL(&ap->addr, addr) &&
-			    (!flags || ap->flags & flags))
+			if (ipv6_findaddrmatch(ap, addr, flags))
 				return ap;
 		}
 	}
@@ -1716,11 +1730,15 @@ dhcp6_findaddr(struct dhcpcd_ctx *ctx, const struct in6_addr *addr,
 {
 	struct interface *ifp;
 	struct ipv6_addr *ap;
+	struct dhcp6_state *state;
 
 	TAILQ_FOREACH(ifp, ctx->ifaces, next) {
-		ap = dhcp6_iffindaddr(ifp, addr, flags);
-		if (ap)
-			return ap;
+		if ((state = D6_STATE(ifp)) != NULL) {
+			TAILQ_FOREACH(ap, &state->addrs, next) {
+				if (ipv6_findaddrmatch(ap, addr, flags))
+					return ap;
+			}
+		}
 	}
 	return NULL;
 }
@@ -1755,7 +1773,10 @@ dhcp6_findna(struct interface *ifp, uint16_t ot, const uint8_t *iaid,
 			continue;
 		}
 		iap = (const struct dhcp6_ia_addr *)D6_COPTION_DATA(o);
-		a = dhcp6_iffindaddr(ifp, &iap->addr, 0);
+		TAILQ_FOREACH(a, &state->addrs, next) {
+			if (ipv6_findaddrmatch(a, &iap->addr, 0))
+				break;
+		}
 		if (a == NULL) {
 			a = calloc(1, sizeof(*a));
 			if (a == NULL) {
@@ -2033,15 +2054,13 @@ dhcp6_findia(struct interface *ifp, const struct dhcp6_message *m, size_t l,
 			continue;
 		}
 		if (code == D6_OPTION_IA_PD) {
-			if (!(ifo->options & DHCPCD_NOPFXDLG) &&
-			    dhcp6_findpd(ifp, iaid, p, ol, acquired) == 0)
-			{
+			if (dhcp6_findpd(ifp, iaid, p, ol, acquired) == 0) {
 				logger(ifp->ctx, LOG_WARNING,
 				    "%s: %s: DHCPv6 REPLY missing Prefix",
 				    ifp->name, sfrom);
 				continue;
 			}
-		} else if (!(ifo->options & DHCPCD_PFXDLGONLY)) {
+		} else {
 			if (dhcp6_findna(ifp, code, iaid, p, ol, acquired) == 0)
 			{
 				logger(ifp->ctx, LOG_WARNING,
@@ -2106,7 +2125,7 @@ dhcp6_validatelease(struct interface *ifp,
 	state->renew = state->rebind = state->expire = 0;
 	state->lowpl = ND6_INFINITE_LIFETIME;
 	if (!acquired) {
-		get_monotonic(&aq);
+		clock_gettime(CLOCK_MONOTONIC, &aq);
 		acquired = &aq;
 	}
 	nia = dhcp6_findia(ifp, m, len, sfrom, acquired);
@@ -2181,7 +2200,7 @@ dhcp6_readlease(struct interface *ifp, int validate)
 	if ((now = time(NULL)) == -1)
 		goto ex;
 
-	get_monotonic(&acquired);
+	clock_gettime(CLOCK_MONOTONIC, &acquired);
 	acquired.tv_sec -= now - st.st_mtime;
 
 	/* Check to see if the lease is still valid */
@@ -2434,8 +2453,6 @@ dhcp6_delegate_prefix(struct interface *ifp)
 	}
 
 	TAILQ_FOREACH(ifd, ifp->ctx->ifaces, next) {
-		if (ifd->options->options & DHCPCD_NOPFXDLG)
-			continue;
 		k = 0;
 		carrier_warned = abrt = 0;
 		TAILQ_FOREACH(ap, &state->addrs, next) {
@@ -2568,24 +2585,6 @@ dhcp6_find_delegates(struct interface *ifp)
 	return k;
 }
 
-static struct interface *
-dhcp6_findpfxdlgif(struct interface *ifp)
-{
-	struct interface *ifn;
-
-	if (ifp->options && ifp->options->options & DHCPCD_PFXDLGONLY)
-		return NULL;
-
-	if (ifp->ctx && ifp->ctx->ifaces) {
-		TAILQ_FOREACH(ifn, ifp->ctx->ifaces, next) {
-			if (strcmp(ifn->name, ifp->name) == 0 &&
-			    ifn->options->options & DHCPCD_PFXDLGONLY)
-				return ifn;
-		}
-	}
-	return NULL;
-}
-
 /* ARGSUSED */
 static void
 dhcp6_handledata(void *arg)
@@ -2596,7 +2595,7 @@ dhcp6_handledata(void *arg)
 	ssize_t bytes;
 	struct cmsghdr *cm;
 	struct in6_pktinfo pkt;
-	struct interface *ifp, *ifpx;
+	struct interface *ifp;
 	const char *op;
 	struct dhcp6_message *r;
 	struct dhcp6_state *state;
@@ -2615,7 +2614,7 @@ dhcp6_handledata(void *arg)
 	if (bytes == -1) {
 		logger(dctx, LOG_ERR, "%s: recvmsg: %m", __func__);
 		close(ctx->dhcp_fd);
-		eloop_event_delete(dctx->eloop, ctx->dhcp_fd, 0);
+		eloop_event_delete(dctx->eloop, ctx->dhcp_fd);
 		ctx->dhcp_fd = -1;
 		return;
 	}
@@ -2649,9 +2648,7 @@ dhcp6_handledata(void *arg)
 	}
 
 	TAILQ_FOREACH(ifp, dctx->ifaces, next) {
-		/* Ensure we work on the master interface */
-		if (ifp->index == (unsigned int)pkt.ipi6_ifindex &&
-		    !(ifp->options->options & DHCPCD_PFXDLGONLY))
+		if (ifp->index == (unsigned int)pkt.ipi6_ifindex)
 			break;
 	}
 	if (ifp == NULL) {
@@ -2661,18 +2658,6 @@ dhcp6_handledata(void *arg)
 		return;
 	}
 
-	r = (struct dhcp6_message *)ctx->rcvhdr.msg_iov[0].iov_base;
-
-	/* Which interface state is the IAID for? */
-	ifpx = dhcp6_findpfxdlgif(ifp);
-	if (ifpx && D6_STATE(ifpx)) {
-		state = D6_STATE(ifpx);
-		if (r->xid[0] == state->send->xid[0] &&
-		    r->xid[1] == state->send->xid[1] &&
-		    r->xid[2] == state->send->xid[2])
-			ifp = ifpx;
-	}
-
 	state = D6_STATE(ifp);
 	if (state == NULL || state->send == NULL) {
 		logger(ifp->ctx, LOG_DEBUG,
@@ -2680,6 +2665,7 @@ dhcp6_handledata(void *arg)
 		return;
 	}
 
+	r = (struct dhcp6_message *)ctx->rcvhdr.msg_iov[0].iov_base;
 	/* We're already bound and this message is for another machine */
 	/* XXX DELEGATED? */
 	if (r->type != DHCP6_RECONFIGURE &&
@@ -3175,43 +3161,6 @@ dhcp6_start1(void *arg)
 	if (dhcp6_findselfsla(ifp, NULL))
 		del_option_mask(ifo->requestmask6, D6_OPTION_RAPID_COMMIT);
 
-	/* Create a 2nd interface to handle the PD state */
-	if (!(ifo->options & (DHCPCD_PFXDLGONLY | DHCPCD_PFXDLGMIX)) &&
-	    dhcp6_hasprefixdelegation(ifp) > 1)
-	{
-		const char * const argv[] = { ifp->name };
-		struct if_head *ifs;
-		struct interface *ifn;
-
-		ifn = dhcp6_findpfxdlgif(ifp);
-		if (ifn == NULL) {
-			ifs = if_discover(ifp->ctx, -1, UNCONST(argv));
-			if (ifs) {
-				ifn = TAILQ_FIRST(ifs);
-				if (ifn) {
-					logger(ifp->ctx, LOG_INFO,
-					    "%s: creating pseudo interface"
-					    " to handle Prefix Delegation",
-					    ifp->name);
-					ifp->options->options |=
-					    DHCPCD_NOPFXDLG;
-					TAILQ_REMOVE(ifs, ifn, next);
-					TAILQ_INSERT_AFTER(ifp->ctx->ifaces,
-					    ifp, ifn, next);
-					dhcpcd_initstate(ifn,
-					    DHCPCD_PFXDLGONLY);
-					eloop_timeout_add_sec(ifp->ctx->eloop,
-					    0, dhcpcd_startinterface, ifn);
-				}
-				while ((ifn = TAILQ_FIRST(ifs))) {
-					TAILQ_REMOVE(ifs, ifn, next);
-					if_free(ifn);
-				}
-				free(ifs);
-			}
-		}
-	}
-
 	if (state->state == DH6S_INFORM) {
 		add_option_mask(ifo->requestmask6, D6_OPTION_INFO_REFRESH_TIME);
 		dhcp6_startinform(ifp);
@@ -3266,8 +3215,7 @@ dhcp6_start(struct interface *ifp, enum DH6S init_state)
 gogogo:
 	state->state = init_state;
 	dhcp_set_leasefile(state->leasefile, sizeof(state->leasefile),
-	    AF_INET6, ifp,
-	    ifp->options->options & DHCPCD_PFXDLGONLY ? ".pd" : "");
+	    AF_INET6, ifp);
 	if (ipv6_linklocal(ifp) == NULL) {
 		logger(ifp->ctx, LOG_DEBUG,
 		    "%s: delaying DHCPv6 soliciation for LL address",
@@ -3304,7 +3252,6 @@ dhcp6_reboot(struct interface *ifp)
 static void
 dhcp6_freedrop(struct interface *ifp, int drop, const char *reason)
 {
-	struct interface *ifpx;
 	struct dhcp6_state *state;
 	struct dhcpcd_ctx *ctx;
 	unsigned long long options;
@@ -3331,18 +3278,7 @@ dhcp6_freedrop(struct interface *ifp, int drop, const char *reason)
 	else
 		options = 0;
 	dropdele = (options & (DHCPCD_STOPPING | DHCPCD_RELEASE) &&
-	    (options &
-	    (DHCPCD_EXITING | DHCPCD_PERSISTENT)) !=
-	    (DHCPCD_EXITING | DHCPCD_PERSISTENT));
-
-	ifpx = dhcp6_findpfxdlgif(ifp);
-	if (ifpx) {
-		if (options & DHCPCD_EXITING)
-			ifpx->options->options |= DHCPCD_EXITING;
-		dhcp6_freedrop(ifpx, dropdele ? 1 : drop, reason);
-		TAILQ_REMOVE(ifp->ctx->ifaces, ifpx, next);
-		if_free(ifpx);
-	}
+	    (options & DHCPCD_NODROP) != DHCPCD_NODROP);
 
 	if (ifp->ctx->eloop)
 		eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
@@ -3352,10 +3288,20 @@ dhcp6_freedrop(struct interface *ifp, int drop, const char *reason)
 
 	state = D6_STATE(ifp);
 	if (state) {
-		dhcp_auth_reset(&state->auth);
+		/* Failure to send the release may cause this function to
+		 * re-enter */
+		if (state->state == DH6S_RELEASE) {
+			dhcp6_finishrelease(ifp);
+			return;
+		}
+
 		if (drop && options & DHCPCD_RELEASE) {
-			if (ifp->carrier == LINK_UP)
+			if (ifp->carrier == LINK_UP &&
+			    state->state != DH6S_RELEASED)
+			{
 				dhcp6_startrelease(ifp);
+				return;
+			}
 			unlink(state->leasefile);
 		}
 		dhcp6_freedrop_addrs(ifp, drop, NULL);
@@ -3365,9 +3311,7 @@ dhcp6_freedrop(struct interface *ifp, int drop, const char *reason)
 		state->new = NULL;
 		state->new_len = 0;
 		if (drop && state->old &&
-		    (options &
-		    (DHCPCD_EXITING | DHCPCD_PERSISTENT)) !=
-		    (DHCPCD_EXITING | DHCPCD_PERSISTENT))
+		    (options & DHCPCD_NODROP) != DHCPCD_NODROP)
 		{
 			if (reason == NULL)
 				reason = "STOP6";
@@ -3391,7 +3335,7 @@ dhcp6_freedrop(struct interface *ifp, int drop, const char *reason)
 	}
 	if (ifp == NULL && ctx->ipv6) {
 		if (ctx->ipv6->dhcp_fd != -1) {
-			eloop_event_delete(ctx->eloop, ctx->ipv6->dhcp_fd, 0);
+			eloop_event_delete(ctx->eloop, ctx->ipv6->dhcp_fd);
 			close(ctx->ipv6->dhcp_fd);
 			ctx->ipv6->dhcp_fd = -1;
 		}
@@ -3588,8 +3532,7 @@ dhcp6_dump(struct interface *ifp)
 	}
 	TAILQ_INIT(&state->addrs);
 	dhcp_set_leasefile(state->leasefile, sizeof(state->leasefile),
-	    AF_INET6, ifp,
-	    ifp->options->options & DHCPCD_PFXDLGONLY ? ".pd" : "");
+	    AF_INET6, ifp);
 	if (dhcp6_readlease(ifp, 0) == -1) {
 		logger(ifp->ctx, LOG_ERR, "%s: %s: %m",
 		    *ifp->name ? ifp->name : state->leasefile, __func__);
