@@ -293,7 +293,7 @@ const char *if_pfname = "Berkley Packet Filter";
 int
 if_openrawsocket(struct interface *ifp, uint16_t protocol)
 {
-	struct dhcp_state *state;
+	struct ipv4_state *state;
 	int fd = -1;
 	struct ifreq ifr;
 	int ibuf_len = 0;
@@ -318,8 +318,7 @@ if_openrawsocket(struct interface *ifp, uint16_t protocol)
 	if (fd == -1)
 		return -1;
 
-	state = D_STATE(ifp);
-
+	state = IPV4_STATE(ifp);
 	memset(&pv, 0, sizeof(pv));
 	if (ioctl(fd, BIOCVERSION, &pv) == -1)
 		goto eexit;
@@ -381,7 +380,6 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 	struct iovec iov[2];
 	struct ether_header hw;
 	int fd;
-	const struct dhcp_state *state;
 
 	memset(&hw, 0, ETHER_HDR_LEN);
 	memset(&hw.ether_dhost, 0xff, ETHER_ADDR_LEN);
@@ -390,11 +388,7 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 	iov[0].iov_len = ETHER_HDR_LEN;
 	iov[1].iov_base = UNCONST(data);
 	iov[1].iov_len = len;
-	state = D_CSTATE(ifp);
-	if (protocol == ETHERTYPE_ARP)
-		fd = state->arp_fd;
-	else
-		fd = state->raw_fd;
+	fd = ipv4_protocol_fd(ifp, protocol);
 	return writev(fd, iov, 2);
 }
 
@@ -408,13 +402,10 @@ if_readrawpacket(struct interface *ifp, uint16_t protocol,
 	struct bpf_hdr packet;
 	ssize_t bytes;
 	const unsigned char *payload;
-	struct dhcp_state *state;
+	struct ipv4_state *state;
 
-	state = D_STATE(ifp);
-	if (protocol == ETHERTYPE_ARP)
-		fd = state->arp_fd;
-	else
-		fd = state->raw_fd;
+	state = IPV4_STATE(ifp);
+	fd = ipv4_protocol_fd(ifp, protocol);
 
 	*flags = 0;
 	for (;;) {
@@ -515,6 +506,7 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct rt_msghdr *rtm)
 	else
 		rt->net.s_addr = INADDR_BROADCAST;
 	COPYOUT(rt->gate, rti_info[RTAX_GATEWAY]);
+	COPYOUT(rt->src, rti_info[RTAX_IFA]);
 
 	if (rtm->rtm_index)
 		rt->iface = if_findindex(ctx->ifaces, rtm->rtm_index);
@@ -1125,7 +1117,7 @@ if_getlifetime6(struct ipv6_addr *ia)
 				    (uint32_t)(lifetime->ia6t_expire -
 				    MIN(t, lifetime->ia6t_expire));
 				/* Calculate the created time */
-				get_monotonic(&ia->created);
+				clock_gettime(CLOCK_MONOTONIC, &ia->created);
 				ia->created.tv_sec -=
 				    lifetime->ia6t_vltime - ia->prefix_vltime;
 			} else
