@@ -351,6 +351,7 @@ get_netlink(struct dhcpcd_ctx *ctx, struct interface *ifp, int fd, int flags,
 				goto eexit;
 			buf = nbuf;
 		}
+		nladdr.nl_pid = 0;
 		nladdr_len = sizeof(nladdr);
 		bytes = recvfrom(fd, buf, buflen, flags,
 		    (struct sockaddr *)&nladdr, &nladdr_len);
@@ -397,7 +398,6 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 	size_t len;
 	struct rtmsg *rtm;
 	struct rtattr *rta;
-	struct in_addr prefsrc;
 
 	len = nlm->nlmsg_len - sizeof(*nlm);
 	if (len < sizeof(*rtm)) {
@@ -414,7 +414,6 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 	if (rtm->rtm_scope == RT_SCOPE_HOST)
 		rt->flags |= RTF_HOST;
 
-	prefsrc.s_addr = INADDR_ANY;
 	rta = (struct rtattr *)RTM_RTA(rtm);
 	len = RTM_PAYLOAD(nlm);
 	while (RTA_OK(rta, len)) {
@@ -428,8 +427,8 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 			    sizeof(rt->gate.s_addr));
 			break;
 		case RTA_PREFSRC:
-			memcpy(&prefsrc.s_addr, RTA_DATA(rta),
-			    sizeof(prefsrc.s_addr));
+			memcpy(&rt->src.s_addr, RTA_DATA(rta),
+			    sizeof(rt->src.s_addr));
 			break;
 		case RTA_OIF:
 			rt->iface = if_findindex(ctx->ifaces,
@@ -443,13 +442,13 @@ if_copyrt(struct dhcpcd_ctx *ctx, struct rt *rt, struct nlmsghdr *nlm)
 	}
 
 	inet_cidrtoaddr(rtm->rtm_dst_len, &rt->net);
-	if (rt->iface == NULL && prefsrc.s_addr != INADDR_ANY) {
+	if (rt->iface == NULL && rt->src.s_addr != INADDR_ANY) {
 		struct ipv4_addr *ap;
 
 		/* For some reason the default route comes back with the
 		 * loopback interface in RTA_OIF? Lets find it by
 		 * preferred source address */
-		if ((ap = ipv4_findaddr(ctx, &prefsrc)))
+		if ((ap = ipv4_findaddr(ctx, &rt->src)))
 			rt->iface = ap->iface;
 	}
 	return 0;
@@ -1196,7 +1195,6 @@ ssize_t
 if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
     const void *data, size_t len)
 {
-	const struct dhcp_state *state;
 	union sockunion {
 		struct sockaddr sa;
 		struct sockaddr_ll sll;
@@ -1215,11 +1213,7 @@ if_sendrawpacket(const struct interface *ifp, uint16_t protocol,
 		    &ipv4_bcast_addr, sizeof(ipv4_bcast_addr));
 	else
 		memset(&su.sll.sll_addr, 0xff, ifp->hwlen);
-	state = D_CSTATE(ifp);
-	if (protocol == ETHERTYPE_ARP)
-		fd = state->arp_fd;
-	else
-		fd = state->raw_fd;
+	fd = ipv4_protocol_fd(ifp, protocol);
 
 	return sendto(fd, data, len, 0, &su.sa, sizeof(su.sll));
 }
@@ -1252,10 +1246,7 @@ if_readrawpacket(struct interface *ifp, uint16_t protocol,
 #endif
 
 	state = D_STATE(ifp);
-	if (protocol == ETHERTYPE_ARP)
-		fd = state->arp_fd;
-	else
-		fd = state->raw_fd;
+	fd = ipv4_protocol_fd(ifp, protocol);
 	bytes = recvmsg(fd, &msg, 0);
 	if (bytes == -1)
 		return -1;
