@@ -35,19 +35,6 @@
 #include <netinet/in.h>
 #include <netinet/if_ether.h>
 
-#ifndef __linux__
-#  ifndef __QNX__
-#    include <sys/endian.h>
-#  endif
-#  include <net/if.h>
-#  ifdef __FreeBSD__ /* Needed so that including netinet6/in6_var.h works */
-#    include <net/if_var.h>
-#  endif
-#  ifndef __sun
-#    include <netinet6/in6_var.h>
-#  endif
-#endif
-
 #include <errno.h>
 #include <ifaddrs.h>
 #include <inttypes.h>
@@ -264,7 +251,8 @@ ipv6_readsecret(struct dhcpcd_ctx *ctx)
 		goto eexit;
 	x = fprintf(fp, "%s\n",
 	    hwaddr_ntoa(ctx->secret, ctx->secret_len, line, sizeof(line)));
-	fclose(fp);
+	if (fclose(fp) == EOF)
+		x = -1;
 	if (x > 0)
 		return (ssize_t)ctx->secret_len;
 
@@ -638,7 +626,7 @@ ipv6_addaddr(struct ipv6_addr *ap, const struct timespec *now)
 
 	/* Ensure no other interface has this address */
 	TAILQ_FOREACH(ifp, ap->iface->ctx->ifaces, next) {
-		if (ifp == ap->iface || strcmp(ifp->name, ap->iface->name) == 0)
+		if (ifp == ap->iface)
 			continue;
 		state = IPV6_STATE(ifp);
 		if (state == NULL)
@@ -685,24 +673,36 @@ ipv6_addaddr(struct ipv6_addr *ap, const struct timespec *now)
 		struct timespec n;
 
 		if (now == NULL) {
-			get_monotonic(&n);
+			clock_gettime(CLOCK_MONOTONIC, &n);
 			now = &n;
 		}
 		timespecsub(now, &ap->acquired, &n);
-		if (ap->prefix_pltime != ND6_INFINITE_LIFETIME)
+		if (ap->prefix_pltime != ND6_INFINITE_LIFETIME) {
 			ap->prefix_pltime -= (uint32_t)n.tv_sec;
+			/* This can happen when confirming a
+			 * deprecated but still valid lease. */
+			if (ap->prefix_pltime > pltime)
+				ap->prefix_pltime = 0;
+		}
 		if (ap->prefix_vltime != ND6_INFINITE_LIFETIME)
 			ap->prefix_vltime -= (uint32_t)n.tv_sec;
-	}
 
-	if (if_addaddress6(ap) == -1) {
-		logger(ap->iface->ctx, LOG_ERR, "if_addaddress6: %m");
 #if 0
+		logger(ap->iface->ctx, LOG_DEBUG,
+		    "%s: acquired %lld.%.9ld, now %lld.%.9ld, diff %lld.%.9ld",
+		    ap->iface->name,
+		    (long long)ap->acquired.tv_sec, ap->acquired.tv_nsec,
+		    (long long)now->tv_sec, now->tv_nsec,
+		    (long long)n.tv_sec, n.tv_nsec);
 		logger(ap->iface->ctx, LOG_DEBUG,
 		    "%s: adj pltime %"PRIu32" seconds, "
 		    "vltime %"PRIu32" seconds",
 		    ap->iface->name, ap->prefix_pltime, ap->prefix_vltime);
 #endif
+	}
+
+	if (if_addaddress6(ap) == -1) {
+		logger(ap->iface->ctx, LOG_ERR, "if_addaddress6: %m");
 		/* Restore real pltime and vltime */
 		ap->prefix_pltime = pltime;
 		ap->prefix_vltime = vltime;
@@ -749,10 +749,28 @@ int
 ipv6_publicaddr(const struct ipv6_addr *ia)
 {
 	return (ia->prefix_pltime &&
-	    (ia->addr.s6_addr[0] & 0xfe) != 0xc &&
+	    (ia->addr.s6_addr[0] & 0xfe) != 0xfc &&
 	    !(ia->addr_flags & IN6_IFF_NOTUSEABLE));
 }
 
+int
+ipv6_findaddrmatch(const struct ipv6_addr *addr, const struct in6_addr *match,
+    short flags)
+{
+
+	if (match == NULL) {
+		if ((addr->flags &
+		    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED)) ==
+		    (IPV6_AF_ADDED | IPV6_AF_DADCOMPLETED))
+			return 1;
+	} else if (addr->prefix_vltime &&
+	    IN6_ARE_ADDR_EQUAL(&addr->addr, match) &&
+	    (!flags || addr->flags & flags))
+		return 1;
+
+	return 0;
+}
+
 struct ipv6_addr *
 ipv6_findaddr(struct dhcpcd_ctx *ctx, const struct in6_addr *addr, short flags)
 {
@@ -799,9 +817,7 @@ ipv6_addaddrs(struct ipv6_addrhead *addrs)
 		{
 			apf = ipv6_findaddr(ap->iface->ctx,
 			    &ap->addr, IPV6_AF_ADDED);
-			if (apf && apf->iface != ap->iface &&
-			    strcmp(apf->iface->name, ap->iface->name))
-			{
+			if (apf && apf->iface != ap->iface) {
 				if (apf->iface->metric <= ap->iface->metric) {
 					logger(apf->iface->ctx, LOG_INFO,
 					    "%s: preferring %s on %s",
@@ -826,7 +842,7 @@ ipv6_addaddrs(struct ipv6_addrhead *addrs)
 			if (ap->flags & IPV6_AF_NEW)
 				i++;
 			if (!timespecisset(&now))
-				get_monotonic(&now);
+				clock_gettime(CLOCK_MONOTONIC, &now);
 			ipv6_addaddr(ap, &now);
 		}
 	}
@@ -865,14 +881,13 @@ ipv6_freedrop_addrs(struct ipv6_addrhead *addrs, int drop,
 			/* Find the same address somewhere else */
 			apf = ipv6_findaddr(ap->iface->ctx, &ap->addr, 0);
 			if (apf == NULL ||
-			    (apf->iface != ap->iface &&
-			    strcmp(apf->iface->name, ap->iface->name)))
+			    (apf->iface != ap->iface))
 				ipv6_deleteaddr(ap);
 			if (!(ap->iface->options->options &
 			    DHCPCD_EXITING) && apf)
 			{
 				if (!timespecisset(&now))
-					get_monotonic(&now);
+					clock_gettime(CLOCK_MONOTONIC, &now);
 				ipv6_addaddr(apf, &now);
 			}
 			if (drop == 2)
@@ -932,109 +947,116 @@ ipv6_handleifa(struct dhcpcd_ctx *ctx,
 		errno = ESRCH;
 		return;
 	}
-	TAILQ_FOREACH(ifp, ifs, next) {
-		/* Each psuedo interface also stores addresses */
-		if (strcmp(ifp->name, ifname))
-			continue;
-		state = ipv6_getstate(ifp);
-		if (state == NULL)
-			continue;
+	if ((ifp = if_find(ifs, ifname)) == NULL)
+		return;
+	if ((state = ipv6_getstate(ifp)) == NULL)
+		return;
 
-		if (!IN6_IS_ADDR_LINKLOCAL(addr)) {
-			ipv6nd_handleifa(ctx, cmd, ifname, addr, flags);
-			dhcp6_handleifa(ctx, cmd, ifname, addr, flags);
-		}
-
-		TAILQ_FOREACH(ap, &state->addrs, next) {
-			if (IN6_ARE_ADDR_EQUAL(&ap->addr, addr))
-				break;
-		}
-
-		switch (cmd) {
-		case RTM_DELADDR:
-			if (ap) {
-				TAILQ_REMOVE(&state->addrs, ap, next);
-				ipv6_freeaddr(ap);
-			}
-			break;
-		case RTM_NEWADDR:
-			if (ap == NULL) {
-				char buf[INET6_ADDRSTRLEN];
-				const char *cbp;
-
-				ap = calloc(1, sizeof(*ap));
-				ap->iface = ifp;
-				ap->addr = *addr;
-				ap->prefix_len = prefix_len;
-				ipv6_makeprefix(&ap->prefix, &ap->addr,
-				    ap->prefix_len);
-				cbp = inet_ntop(AF_INET6, &addr->s6_addr,
-				    buf, sizeof(buf));
-				if (cbp)
-					snprintf(ap->saddr, sizeof(ap->saddr),
-					    "%s/%d", cbp, prefix_len);
-				if (if_getlifetime6(ap) == -1) {
-					/* No support or address vanished.
-					 * Either way, just set a deprecated
-					 * infinite time lifetime and continue.
-					 * This is fine because we only want
-					 * to know this when trying to extend
-					 * temporary addresses.
-					 * As we can't extend infinite, we'll
-					 * create a new temporary address. */
-					ap->prefix_pltime = 0;
-					ap->prefix_vltime =
-					    ND6_INFINITE_LIFETIME;
-				}
-				/* This is a minor regression against RFC 4941
-				 * because the kernel only knows when the
-				 * lifetimes were last updated, not when the
-				 * address was initially created.
-				 * Provided dhcpcd is not restarted, this
-				 * won't be a problem.
-				 * If we don't like it, we can always
-				 * pretend lifetimes are infinite and always
-				 * generate a new temporary address on
-				 * restart. */
-				ap->acquired = ap->created;
-				TAILQ_INSERT_TAIL(&state->addrs,
-				    ap, next);
-			}
-			ap->addr_flags = flags;
-#ifdef IPV6_MANAGETEMPADDR
-			if (ap->addr_flags & IN6_IFF_TEMPORARY)
-				ap->flags |= IPV6_AF_TEMPORARY;
-#endif
-			if (IN6_IS_ADDR_LINKLOCAL(&ap->addr)) {
-#ifdef IPV6_POLLADDRFLAG
-				if (ap->addr_flags & IN6_IFF_TENTATIVE) {
-					struct timespec tv;
-
-					ms_to_ts(&tv, RETRANS_TIMER / 2);
-					eloop_timeout_add_tv(
-					    ap->iface->ctx->eloop,
-					    &tv, ipv6_checkaddrflags, ap);
-					break;
-				}
-#endif
-
-				if (!(ap->addr_flags & IN6_IFF_NOTUSEABLE)) {
-					/* Now run any callbacks.
-					 * Typically IPv6RS or DHCPv6 */
-					while ((cb =
-					    TAILQ_FIRST(&state->ll_callbacks)))
-					{
-						TAILQ_REMOVE(
-						    &state->ll_callbacks,
-						    cb, next);
-						cb->callback(cb->arg);
-						free(cb);
-					}
-				}
-			}
-			break;
-		}
+	if (!IN6_IS_ADDR_LINKLOCAL(addr)) {
+		ipv6nd_handleifa(ctx, cmd, ifname, addr, flags);
+		dhcp6_handleifa(ctx, cmd, ifname, addr, flags);
 	}
+
+	TAILQ_FOREACH(ap, &state->addrs, next) {
+		if (IN6_ARE_ADDR_EQUAL(&ap->addr, addr))
+			break;
+	}
+
+	switch (cmd) {
+	case RTM_DELADDR:
+		if (ap) {
+			TAILQ_REMOVE(&state->addrs, ap, next);
+			ipv6_freeaddr(ap);
+		}
+		break;
+	case RTM_NEWADDR:
+		if (ap == NULL) {
+			char buf[INET6_ADDRSTRLEN];
+			const char *cbp;
+
+			ap = calloc(1, sizeof(*ap));
+			ap->iface = ifp;
+			ap->addr = *addr;
+			ap->prefix_len = prefix_len;
+			ipv6_makeprefix(&ap->prefix, &ap->addr,
+			    ap->prefix_len);
+			cbp = inet_ntop(AF_INET6, &addr->s6_addr,
+			    buf, sizeof(buf));
+			if (cbp)
+				snprintf(ap->saddr, sizeof(ap->saddr),
+				    "%s/%d", cbp, prefix_len);
+			if (if_getlifetime6(ap) == -1) {
+				/* No support or address vanished.
+				 * Either way, just set a deprecated
+				 * infinite time lifetime and continue.
+				 * This is fine because we only want
+				 * to know this when trying to extend
+				 * temporary addresses.
+				 * As we can't extend infinite, we'll
+				 * create a new temporary address. */
+				ap->prefix_pltime = 0;
+				ap->prefix_vltime =
+				    ND6_INFINITE_LIFETIME;
+			}
+			/* This is a minor regression against RFC 4941
+			 * because the kernel only knows when the
+			 * lifetimes were last updated, not when the
+			 * address was initially created.
+			 * Provided dhcpcd is not restarted, this
+			 * won't be a problem.
+			 * If we don't like it, we can always
+			 * pretend lifetimes are infinite and always
+			 * generate a new temporary address on
+			 * restart. */
+			ap->acquired = ap->created;
+			TAILQ_INSERT_TAIL(&state->addrs,
+			    ap, next);
+		}
+		ap->addr_flags = flags;
+#ifdef IPV6_MANAGETEMPADDR
+		if (ap->addr_flags & IN6_IFF_TEMPORARY)
+			ap->flags |= IPV6_AF_TEMPORARY;
+#endif
+		if (IN6_IS_ADDR_LINKLOCAL(&ap->addr)) {
+#ifdef IPV6_POLLADDRFLAG
+			if (ap->addr_flags & IN6_IFF_TENTATIVE) {
+				struct timespec tv;
+
+				ms_to_ts(&tv, RETRANS_TIMER / 2);
+				eloop_timeout_add_tv(
+				    ap->iface->ctx->eloop,
+				    &tv, ipv6_checkaddrflags, ap);
+				break;
+			}
+#endif
+
+			if (!(ap->addr_flags & IN6_IFF_NOTUSEABLE)) {
+				/* Now run any callbacks.
+				 * Typically IPv6RS or DHCPv6 */
+				while ((cb =
+				    TAILQ_FIRST(&state->ll_callbacks)))
+				{
+					TAILQ_REMOVE(
+					    &state->ll_callbacks,
+					    cb, next);
+					cb->callback(cb->arg);
+					free(cb);
+				}
+			}
+		}
+		break;
+	}
+}
+
+int
+ipv6_hasaddr(const struct interface *ifp)
+{
+
+	if (ipv6nd_iffindaddr(ifp, NULL, 0) != NULL)
+		return 1;
+	if (dhcp6_iffindaddr(ifp, NULL, 0) != NULL)
+		return 1;
+	return 0;
 }
 
 const struct ipv6_addr *
@@ -1463,7 +1485,7 @@ ipv6_tempdadcallback(void *arg)
 			    ia->iface->name);
 			return;
 		}
-		get_monotonic(&tv);
+		clock_gettime(CLOCK_MONOTONIC, &tv);
 		if ((ia1 = ipv6_createtempaddr(ia, &tv)) == NULL)
 			logger(ia->iface->ctx, LOG_ERR,
 			    "ipv6_createtempaddr: %m");
@@ -1568,9 +1590,7 @@ again:
 	cbp = inet_ntop(AF_INET6, &ia->addr, buf, sizeof(buf));
 	if (cbp)
 		snprintf(ia->saddr, sizeof(ia->saddr), "%s/%d",
-		    cbp, ia->prefix_len);
-	else
-		ia->saddr[0] = '\0';
+		    cbp, ia->prefix_len); else ia->saddr[0] = '\0';
 
 	TAILQ_INSERT_TAIL(&state->addrs, ia, next);
 	return ia;
@@ -1689,7 +1709,7 @@ ipv6_regentempaddr(void *arg)
 
 	logger(ia->iface->ctx, LOG_DEBUG, "%s: regen temp addr %s",
 	    ia->iface->name, ia->saddr);
-	get_monotonic(&tv);
+	clock_gettime(CLOCK_MONOTONIC, &tv);
 	ia1 = ipv6_createtempaddr(ia, &tv);
 	if (ia1)
 		ipv6_addaddr(ia1, &tv);
